
# q-expansion of newform 6468.2.a.bf, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6468
weight = 2
poly_data = [-28, -16, 58, 20, -15, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[16, -28, -26, -7, 2, 1], 18], [[-14, -16, 43, 23, -4, -2], 9], [[-34, -26, 26, 13, -2, -1], 6], [[40, -52, -47, 23, 5, -2], 9]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [1, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 1], [1, 0, 0, 0, 1, 0], [1, 0, 0, -1, 0, -1], [2, -1, -1, -1, 0, -1], [0, 0, 1, 1, 1, 1], [2, -1, -1, 1, 0, -1], [1, -2, 0, 1, 0, 1], [1, 1, -2, 0, -1, -1], [1, 2, 3, 0, -1, 0], [0, 2, 1, -1, -1, 1], [0, -1, 1, 1, -2, 1], [3, 1, 2, 1, -2, 0], [-1, 1, 3, 1, 1, 1], [0, -1, -3, 1, -1, -2], [2, 0, 3, -1, 0, 0], [2, 3, 1, 2, -1, 2], [-2, 0, 0, 2, -2, 0], [0, 3, -1, -1, 0, 1], [0, 2, 2, 1, 0, 0], [4, -1, -2, -1, 2, -1], [3, 0, 2, 2, -1, 0], [-1, -3, 2, 1, -1, -1], [3, 1, -4, -1, 0, -2], [0, 1, -3, -1, 2, 1], [2, 0, 5, 3, -2, 0], [1, 0, 7, 0, 1, 0], [-4, 0, -3, -3, 2, 0], [-2, 1, 1, 1, 2, -1], [5, -1, 1, 0, -1, -3], [2, -2, -5, -1, 1, -1], [-2, 1, -3, -3, 0, 1], [9, -1, -7, -3, 1, -3], [-6, 3, -3, -1, 3, 0], [-2, 1, 1, 5, 0, 1], [5, 3, 1, -1, 1, -3], [4, 4, -4, 0, 0, 0], [6, -3, 0, 1, -4, 1], [4, 1, 0, 0, -2, 3], [-2, -3, -3, -1, -2, -3], [-1, 1, 7, 0, 3, -1], [3, 2, -7, -2, 1, 0], [-8, 6, 6, -2, 2, 0], [4, 2, 10, 0, 2, 0], [6, 0, 1, -1, 2, -2], [13, -4, 2, 0, -3, 0], [-1, 3, -5, -6, 3, -1], [1, 5, 8, 1, -2, 2], [3, 0, -2, 2, -4, 3], [5, 3, 4, 1, 0, 2], [7, -1, 2, 6, 0, 2], [12, 0, -7, 1, -1, 3], [11, -1, 1, -3, 1, -1], [-4, 5, 1, -3, 1, 2], [-15, 3, -2, 3, -1, 1], [5, -7, 4, 1, 0, 0], [10, -1, -1, 3, -3, 2], [11, -1, -3, 3, -1, 1], [4, -2, 8, 2, -2, 0], [1, 1, -3, 0, 3, 3], [7, 2, 6, 0, 5, 0], [-4, 3, -6, 4, -4, 1], [-14, 0, 8, 2, 2, 0], [-10, 3, -1, -5, -2, -3], [3, 3, -9, -2, 3, -1], [6, -3, 1, 0, 3, 0], [10, -4, 3, -2, -1, -5], [1, 7, 1, 2, -1, 5], [2, 4, -4, -4, 0, 2], [-9, 2, -3, -2, -1, 0], [-3, -2, 6, 3, 0, 1], [7, 3, -9, -4, -1, -1], [2, -4, -3, -3, 0, -4], [1, 2, 8, 4, -1, 0], [-2, 1, -6, 0, -2, -3], [8, 0, -2, 1, 2, 0], [8, 0, -1, -5, 1, -3], [1, 2, 0, -5, 4, -1], [-1, -5, -10, 1, 2, -2], [5, 1, -9, -5, 1, -1], [-16, 1, -7, -3, -3, -2], [4, 2, 7, 1, -2, -2], [-4, 4, -3, -3, 2, -2], [-10, -5, -5, 1, -2, -1], [-3, -1, 8, 0, 3, -3], [-3, -4, 2, -5, 0, 3], [3, 3, -4, 3, -6, 2], [-6, 5, 2, 0, -4, -1], [2, -4, -2, 0, -4, -6], [-7, 7, 4, 1, 2, 0], [-8, -7, -3, -3, 1, -2], [-8, -3, -10, 2, 0, -3], [4, 0, -6, -1, 2, 4], [3, -7, 8, 0, 2, 2], [-5, -2, 2, 3, 0, -1], [-17, 9, 6, 1, 1, 3], [-15, -1, 0, 3, 3, -5], [-12, -2, 1, -5, 3, 3], [-8, -3, 10, 4, 0, 1], [9, -3, -11, 2, 3, 3], [-1, 1, -20, 1, 1, 1], [11, 0, -4, 4, 1, -2], [8, -2, -3, -3, -3, 3], [-5, -1, -12, -6, 1, 1], [10, -6, 4, 2, -6, 2], [-5, 8, -2, 4, 0, 3], [-16, -1, -3, -7, 1, 0], [-7, -8, 7, -4, 5, -4], [-2, 4, 8, 6, 0, 6], [-9, 4, -7, 8, -1, 4], [-2, -8, 8, 2, -2, -4], [24, -1, 3, -3, 0, 1], [3, -4, 1, 0, 5, 2], [6, -2, 5, 1, -1, -1], [0, -2, 6, 2, 0, 2], [11, 5, 0, -5, -2, 4], [-17, 8, -6, 2, 3, 4], [-5, 8, 1, 4, 1, 2], [1, -1, 2, -2, 1, -1], [-2, 1, -12, 2, -4, 3], [6, 4, -8, -6, 4, -2], [-4, -4, -10, 2, -2, 2], [-7, -2, -8, 1, 4, 5], [-5, -3, -2, -1, 8, -2], [-6, 3, 7, -3, -2, -3], [11, -5, -1, -9, 3, -7], [-4, 1, -13, -3, 0, 3], [16, -4, 1, -7, -1, -1], [-16, -1, 5, -3, -1, 2], [6, 5, -11, -5, 5, 4], [-5, 1, -9, -1, -5, -1], [-4, -1, 9, -2, 1, -2], [10, -6, 5, 6, -9, 1], [6, -11, -3, 1, -1, -2], [-2, 2, -3, 0, 1, 7], [3, -5, 4, 5, 0, 2], [4, 7, 1, -5, -1, 2], [4, -2, 7, 1, -3, -3], [-1, -6, 14, 1, 4, -1], [-24, 4, -3, 1, -1, -1], [-9, 1, -15, 1, -5, -5], [-2, -4, 9, 1, -1, 5], [-3, 5, 7, 1, -1, -3], [-7, -3, -3, -1, 3, -7], [-10, -4, 2, 4, 2, 4], [6, -7, 2, 8, -2, 5], [-16, 0, -12, -4, 0, -4], [0, -12, -11, 0, -3, -9], [3, 8, -4, -1, -8, 1], [-6, -1, 18, -4, 0, 5], [-5, -5, -1, -10, 1, -3], [14, 4, -13, -5, 2, -6], [-6, -1, -1, -9, 6, 1], [20, -4, -19, 0, 3, -3], [1, 1, -13, 3, -3, -5], [-19, 3, -5, -1, 1, 5], [6, 4, 10, 0, 4, 2], [1, -1, -10, 1, -4, -8], [-3, -5, 4, 7, 1, 1], [-1, -3, 8, 3, -2, -4], [0, -1, 31, -3, 2, -1], [-7, 7, -5, -10, 7, 1], [-5, 2, -10, 5, 0, -5], [-6, 12, -4, -1, -2, -2], [1, 7, -12, 5, -3, 7], [-4, -2, -5, 4, -5, -1], [-2, 6, 1, -1, -7, -3], [3, -3, 11, -1, 1, 1], [18, -2, -14, -8, 4, -8], [-2, 9, -7, -3, -4, 1], [2, -2, -4, -4, -4, 8], [0, -4, -2, 3, 0, 6], [-6, -5, 5, 9, 2, 1], [-24, 8, 7, -7, 4, 2], [-26, 5, -3, -1, 7, 2], [-9, 6, 16, 0, 4, 1], [-19, 6, -8, 1, 4, -3], [-18, -3, 5, 3, 0, -7], [6, 6, -14, -4, 0, -4], [31, -9, 12, -4, -3, -3], [10, 2, -2, -4, 10, -2], [9, -1, -2, 6, 4, 0], [-8, 4, 8, -2, -8, 6], [-33, 10, -13, 2, -1, -2], [-7, 16, -3, -2, 5, 0], [-16, -9, -1, 1, -4, -3], [31, 3, 13, -1, 1, 3], [20, 2, -15, 1, -1, 5], [7, 4, 10, 1, -10, 3], [-9, 9, 4, 8, -8, 8], [-5, -1, -18, 1, 2, -6], [6, 4, 19, 0, -7, 1], [-12, -3, -5, 5, 2, -3], [0, -4, -10, -2, 2, -2], [5, -10, -8, 4, 3, 0], [-32, -3, -1, -3, 2, 5], [2, 2, 28, -4, 4, -2], [-15, -8, 14, 7, 2, 5], [33, -6, 18, -2, -3, -2], [-20, -5, -5, 4, -3, 2], [9, -3, -5, 0, 1, -7], [-8, 0, 8, -5, 4, 6], [-1, -5, -14, -3, 1, -5], [-3, 7, 0, -3, -2, 0], [-15, 15, -1, -2, 9, 5], [-14, -13, -5, -1, 2, -3], [-1, -11, -2, -1, 3, -1], [-4, -6, -1, -4, 9, -3], [12, -1, -3, 5, 5, 4], [42, -4, -8, 2, 0, -2], [-32, -10, -11, -5, 4, 4], [-33, 9, 3, -7, 5, 1], [-29, 5, 20, 9, 1, 9], [7, -10, -10, -2, -9, -6], [16, 3, -1, 1, -4, 5], [-4, 1, -16, 8, -2, 3], [-21, 6, 15, -6, -1, 6], [-23, 0, 6, 11, -6, 5], [14, -9, -3, -1, 0, 1], [-18, 3, -7, -1, -1, -4], [-2, 4, -18, -6, -6, -4], [11, 5, 12, 5, -1, -5], [10, -14, -17, 0, -1, -3], [16, -2, 26, -2, -2, 0], [-20, -3, -15, 1, 5, 6], [11, -5, 32, -1, 0, 0], [3, 5, 9, 8, 3, 5], [-23, 20, 2, 1, 4, 3], [-31, -1, -8, -5, 1, 1], [9, 5, -6, 5, -6, -4], [14, -7, 5, -3, -6, -5], [-24, 4, 9, 3, -8, 6], [2, 1, 6, -5, 6, 3], [3, -7, -2, 3, 0, -4], [20, -1, -21, 1, 0, -7], [-24, 4, -5, 9, 1, 5], [22, -1, 13, 5, 4, 3], [-30, 6, -6, -2, -2, -2], [-7, -4, -5, -2, 1, 0], [17, -7, -21, -2, -3, -9], [5, 7, 3, -9, 5, 5], [6, 10, 14, 0, 2, 6], [-6, 6, 2, 6, 6, -6], [7, 9, 0, 3, -6, -4], [-7, 3, -18, 5, 3, -5], [-10, 4, 15, 9, -3, 3], [17, 7, -14, -9, -7, -1], [-15, 0, 5, -3, -3, 2], [-18, 2, -4, -6, 0, 0], [-23, 3, 15, 11, -5, 11], [-11, -5, -2, -4, 5, -1], [-15, 5, 15, -8, -1, 7], [-2, 3, -13, -3, -3, -10], [-20, 3, 11, -7, 7, -6], [6, 0, -11, -8, 3, 3], [18, -7, 5, 4, -9, 4], [6, -10, -14, -8, -4, -8], [24, 0, -12, -4, 2, -6], [37, -7, -5, 11, 1, 3], [-4, 1, 15, 4, -1, 6], [5, -9, -4, 0, 12, 0], [30, -1, -3, -3, -8, 3], [-38, 10, -12, 4, 4, -2], [-3, 1, 24, -8, -2, 0], [-28, 4, -10, -2, 8, 8], [7, -9, -9, 2, 5, 1], [-3, -9, -5, 7, -3, 9], [-17, 6, 32, 3, -4, 7], [1, 10, -2, -4, 9, 6], [5, -8, 15, -4, 5, 0], [-4, -19, -11, -2, 11, -6], [-39, 4, -7, -4, 1, -4], [-10, -2, 2, -12, 6, 2], [6, -2, 20, 4, 0, -2], [-11, 0, 7, 2, 1, -2], [-24, -5, 17, 5, -2, 7], [-4, 13, 3, 9, -2, 5], [10, -14, 6, 8, -2, 2], [4, -9, -6, -2, 6, -3], [-1, 2, -8, -14, 12, 1], [42, -3, -21, -9, 9, -12], [-6, 5, -2, -12, -6, -7], [19, -2, 4, -7, 10, -1], [1, -5, -5, 2, 13, 5], [19, 0, 9, -2, -5, -4], [11, -6, -11, 5, 1, 2], [2, -7, 21, -1, 2, 5], [-7, -7, 17, 6, -7, 1], [-2, 0, 13, 7, -13, 1], [6, -1, -16, 0, 2, -9], [14, -10, -14, 10, -2, -4], [11, -2, 1, -13, 7, -12], [-24, -14, -28, -6, 2, 0], [-1, 3, 17, -9, -5, -7], [-18, 5, -4, 0, -6, -3], [-3, 0, -4, -9, 2, 1], [7, -12, 24, 6, -2, 1], [-20, -4, -1, 3, -5, 3], [-9, -2, 23, -6, 1, 0], [-20, 11, 8, 8, 4, 7], [-23, 2, 12, 9, -8, 13], [-11, 10, -4, 9, -4, 13], [12, -5, 6, 2, -12, -1], [14, -5, 0, -13, -4, -13], [-14, -3, -16, 2, -2, -3], [18, -13, 7, 9, -2, 3], [-26, 10, -3, -3, -6, -4], [0, 8, -8, -8, -4, -12], [-4, -22, 9, 1, 3, -3], [1, -9, 9, 14, -15, -1], [7, 15, -3, -4, -3, -11], [-16, -8, 2, 5, -6, 0], [22, 8, 20, -2, 12, 2], [6, 2, -34, 4, -2, 2], [-13, 12, -14, -4, -9, -4], [16, -10, -14, -6, 2, -12], [19, -8, 24, 1, 4, 1], [28, -5, -25, 3, 3, 2], [19, 7, -13, -1, -3, 3], [-1, -19, -9, 2, 1, -9], [31, -15, 18, 3, 0, 0], [1, -9, 11, 9, -7, -3], [-11, -5, 13, -2, -3, -9], [-1, 14, -5, 2, 1, 8], [-1, -3, -34, -3, 0, 2], [-37, -13, 6, 1, 2, -10], [-9, 17, 6, -3, 9, 7], [-34, 10, 12, 6, -6, 10], [7, -10, 22, 9, -10, 1], [0, -3, -9, 3, 4, 15], [-9, -2, -11, -4, 9, 4], [-1, 1, -18, 5, 2, 16], [-4, 3, -1, -1, -3, 8], [-5, -2, 3, 11, 3, 8], [9, 0, -8, 2, 7, 10], [-12, 6, -24, -4, 14, 4], [14, -2, -6, -2, 8, -6], [-4, -5, -1, 5, -8, 9], [-17, -14, 5, 1, -3, -6], [-18, 11, 15, -11, -5, 0], [-8, 23, 13, 3, 4, 9], [17, -13, 12, 13, -10, -8], [48, -12, -3, 0, 0, -4], [1, 15, 5, -2, 7, 13], [-14, 12, 2, 0, 6, 16], [-2, 18, 28, 1, 6, 8], [-18, -8, -26, -10, -6, -8], [-11, 2, -10, 9, -12, -1], [-56, 16, 11, -1, 2, 6], [-22, 21, -1, 5, 3, 6], [27, -12, 7, 4, -13, -6], [-3, 12, -1, -5, 11, -8], [-19, -3, 16, 3, -10, 10], [-16, -10, 4, -18, 8, -6], [5, 5, -12, -5, -8, 6], [30, 5, 9, 1, 8, 1], [-5, 3, 24, 11, -3, 1], [-57, 3, -13, 1, 7, 5], [-26, -10, 17, 1, 8, -2], [12, 10, 11, -5, 1, -5], [18, 6, 16, -2, -2, -6], [17, 14, -24, -7, 0, -7], [21, -8, 20, -4, -7, -4], [-7, 19, -2, 9, -3, 9], [23, -13, 18, -11, 11, -1], [-16, 7, -40, 4, 2, 5], [58, 5, -13, 3, -4, 5], [-21, -1, -8, 3, 0, 4], [43, -7, 0, 1, -4, 4], [-4, 8, 20, 0, -12, 0], [8, 11, -7, -1, -7, -2], [26, -12, -7, -13, 5, -9], [-13, 15, 28, 7, 2, 10], [2, 5, -1, -12, 11, 4], [5, 3, -4, 8, -7, -5], [-7, -11, -35, -3, -7, -1], [-10, 14, -15, -2, -5, 5], [-20, 15, 6, -12, 0, -3], [-9, 5, 8, -11, -1, -1], [26, -8, -7, 1, -1, -13], [10, -2, 0, 0, 10, -2], [5, -5, -14, -7, 14, -4], [-14, 7, 13, -1, -7, 6], [15, 11, 11, 5, -3, 11], [14, -9, 19, -5, 0, 3], [16, 6, -19, -5, -1, -11], [40, 3, -7, -3, 3, -14], [22, 0, -12, 3, -2, -6], [-6, 7, -18, -8, 10, -1], [17, -2, -13, -8, -9, -8], [-12, 6, -23, 7, -4, -4], [31, 0, -15, 4, -1, 0], [8, 5, -15, -15, 11, -4], [35, 3, 20, -8, 10, 4], [-32, -18, -10, -6, 8, -2], [-16, -2, -53, -5, 3, 1], [-11, -20, 16, 10, 0, -3], [4, -1, -20, -2, 6, -9], [-20, 13, -15, -1, -10, -7], [45, 5, -1, 9, -1, 9], [11, 8, -21, -5, -1, -2], [-17, 8, -26, 9, -2, 3], [-6, -3, 19, 3, 1, -10], [40, 4, 10, 4, -2, 0], [32, 0, 4, 0, -2, -2], [35, 1, 23, -6, 1, -5], [-27, 9, 3, -15, -1, 1], [2, 0, -2, 8, 10, -4], [-34, 0, -5, -1, 8, 4], [24, -3, 19, 13, 8, 3], [-29, -3, 0, 9, 11, -9], [-42, -6, 15, -2, 1, -5], [18, 16, -17, 9, -9, 9], [11, -4, 19, 4, -9, 8], [10, -12, -10, -1, 10, -10], [-14, -11, 28, 2, 0, -1], [2, 1, -3, -7, 2, -13], [4, -21, 13, 6, 11, 8], [0, -3, 20, 8, 2, -5], [38, -23, 11, 5, 6, 3], [7, -11, -27, 7, -3, 1], [23, 3, 0, 4, -10, -4], [-2, 13, -12, -10, -4, -1], [-2, -16, 25, 7, -3, 7], [-15, 21, 2, -3, 5, 3], [14, 2, -39, 5, 0, 6]]
